==> sm-0.0.1.tar.gz: sm/tests/src/Kernel/SmSendersLocatorTest.php <==
<?php

declare(strict_types = 1);

namespace Drupal\Tests\sm\Kernel;

use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\KernelTests\KernelTestBase;
use Drupal\sm_test\SmTestMessage;
use Symfony\Component\DependencyInjection\Reference;
use Symfony\Component\DependencyInjection\ServiceLocator;
use Symfony\Component\Messenger\Envelope;
use Symfony\Component\Messenger\Transport\InMemory\InMemoryTransport;
use Symfony\Component\Messenger\Transport\Sender\SendersLocator;
use Symfony\Component\Messenger\Transport\Sender\SendersLocatorInterface;

/**
 * Tests senders locator.
 *
 * Tests 'messenger.senders_locator' service.
 *
 * @covers \Symfony\Component\Messenger\Transport\Sender\SendersLocator
 * @covers \Symfony\Component\Messenger\Transport\Sender\SendersLocatorInterface
 * @covers \Drupal\sm\SmCompilerPass::sendersLocator
 */
final class SmSendersLocatorTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'sm',
    'sm_test_transport',
  ];

  /**
   * @var null|array
   */
  private ?array $messageToSendersMapping = NULL;

  /**
   * Tests senders locator.
   */
  public function testSendersLocator(): void {
    // Empty locator.
    $definition = $this->container->getDefinition(SendersLocatorInterface::class);
    $this->assertEquals([], $definition->getArgument(0));

    // This can be a Reference or Definition somehow depending on the transports
    // in the system. Not sure why. Maybe quantity?
    $arg1 = $definition->getArgument(1);
    $this->assertInstanceOf(Reference::class, $arg1);
    $serviceLocator = $this->container->get((string) $arg1);
    $this->assertInstanceOf(ServiceLocator::class, $serviceLocator);
    $this->assertEquals([
      'sm_test_transport.transport.sm_test_transport__in_memory',
      'sm_test_transport.transport.sm_test_transport__in_memory2',
      'sm_test_transport__in_memory',
      'sm_test_transport__in_memory2',
    ], array_keys($serviceLocator->getProvidedServices()));

    $sendersLocator = \Drupal::getContainer()->get(SendersLocatorInterface::class);
    static::assertInstanceOf(SendersLocator::class, $sendersLocator);
    $envelope = new Envelope(new SmTestMessage());
    $senders = iterator_to_array($sendersLocator->getSenders($envelope));
    $this->assertCount(0, $senders);

    // Fallback locator.
    $this->setRouting(['*' => 'sm_test_transport__in_memory']);
    $definition = $this->container->getDefinition(SendersLocatorInterface::class);
    $this->assertEquals([
      '*' => ['sm_test_transport__in_memory'],
    ], $definition->getArgument(0));

    $arg1 = $definition->getArgument(1);
    $this->assertInstanceOf(Reference::class, $arg1);
    $serviceLocator = $this->container->get((string) $arg1);
    $this->assertInstanceOf(ServiceLocator::class, $serviceLocator);
    $this->assertEquals([
      'sm_test_transport.transport.sm_test_transport__in_memory',
      'sm_test_transport.transport.sm_test_transport__in_memory2',
      'sm_test_transport__in_memory',
      'sm_test_transport__in_memory2',
    ], array_keys($serviceLocator->getProvidedServices()));

    $sendersLocator = \Drupal::getContainer()->get(SendersLocatorInterface::class);
    static::assertInstanceOf(SendersLocator::class, $sendersLocator);
    $envelope = new Envelope(new SmTestMessage());
    $senders = iterator_to_array($sendersLocator->getSenders($envelope));
    $this->assertCount(1, $senders);
    $this->assertArrayHasKey('sm_test_transport__in_memory', $senders);
    $this->assertInstanceOf(InMemoryTransport::class, $senders['sm_test_transport__in_memory']);
  }

  /**
   * Tests matching on partial class name.
   *
   * This is functionality provided by Symfony Messenger.
   */
  public function testSendersLocatorPartialClassMatch(): void {
    $this->setRouting([
      'Drupal\sm_test\*' => 'sm_test_transport__in_memory2',
      '*' => 'sm_test_transport__in_memory',
    ]);

    $sendersLocator = \Drupal::getContainer()->get(SendersLocatorInterface::class);
    $envelope = new Envelope(new SmTestMessage());
    $senders = iterator_to_array($sendersLocator->getSenders($envelope));
    $this->assertCount(1, $senders);
    $this->assertArrayHasKey('sm_test_transport__in_memory2', $senders);
  }

  /**
   * Tests matching on exact class name.
   *
   * This is functionality provided by Symfony Messenger.
   */
  public function testSendersLocatorExactClassMatch(): void {
    $this->setRouting([
      'Drupal\sm_test\SmTestMessage' => 'sm_test_transport__in_memory2',
      'Drupal\sm_test' => 'sm_test_transport__in_memory',
      '*' => 'sm_test_transport__in_memory',
    ]);

    $sendersLocator = \Drupal::getContainer()->get(SendersLocatorInterface::class);
    $envelope = new Envelope(new SmTestMessage());
    $senders = iterator_to_array($sendersLocator->getSenders($envelope));
    $this->assertCount(1, $senders);
    $this->assertArrayHasKey('sm_test_transport__in_memory2', $senders);
  }

  /**
   * {@inheritdoc}
   */
  public function register(ContainerBuilder $container): void {
    // This method can be invoked again when calling kernel->rebuildContainer
    // a strategy borrowed from `DrupalFlushAllCachesTest`.
    parent::register($container);

    if (NULL !== $this->messageToSendersMapping) {
      $container->setParameter('sm.routing', $this->messageToSendersMapping);
    }
  }

  /**
   * Sets routing and triggers a rebuild.
   *
   * @phpstan-param array<string|class-string, string|string[]> $messageToSendersMapping
   */
  private function setRouting(array $messageToSendersMapping): void {
    $this->messageToSendersMapping = $messageToSendersMapping;
    \Drupal::service('kernel')->rebuildContainer();
  }

}

==> sm-0.0.2.tar.gz: sm/tests/src/Kernel/SmSendersLocatorTest.php <==
<?php

declare(strict_types = 1);

namespace Drupal\Tests\sm\Kernel;

use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\KernelTests\KernelTestBase;
use Drupal\sm_test\SmTestMessage;
use Symfony\Component\DependencyInjection\Reference;
use Symfony\Component\DependencyInjection\ServiceLocator;
use Symfony\Component\Messenger\Envelope;
use Symfony\Component\Messenger\Transport\InMemory\InMemoryTransport;
use Symfony\Component\Messenger\Transport\Sender\SendersLocator;
use Symfony\Component\Messenger\Transport\Sender\SendersLocatorInterface;

/**
 * Tests senders locator.
 *
 * Tests 'messenger.senders_locator' service.
 *
 * @covers \Symfony\Component\Messenger\Transport\Sender\SendersLocator
 * @covers \Symfony\Component\Messenger\Transport\Sender\SendersLocatorInterface
 * @covers \Drupal\sm\SmCompilerPass::sendersLocator
 */
final class SmSendersLocatorTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'sm',
    'sm_test_transport',
  ];

  /**
   * @var null|array
   */
  private ?array $messageToSendersMapping = NULL;

  /**
   * Tests senders locator.
   */
  public function testSendersLocator(): void {
    // Empty locator.
    $definition = $this->container->getDefinition(SendersLocatorInterface::class);
    static::assertEquals([], $definition->getArgument(0));

    // This can be a Reference or Definition somehow depending on the transports
    // in the system. Not sure why. Maybe quantity?
    $arg1 = $definition->getArgument(1);
    static::assertInstanceOf(Reference::class, $arg1);
    $serviceLocator = $this->container->get((string) $arg1);
    static::assertInstanceOf(ServiceLocator::class, $serviceLocator);
    static::assertEquals([
      'sm_test_transport.transport.sm_test_transport__in_memory',
      'sm_test_transport.transport.sm_test_transport__in_memory2',
      'sm_test_transport__in_memory',
      'sm_test_transport__in_memory2',
    ], array_keys($serviceLocator->getProvidedServices()));

    $sendersLocator = \Drupal::getContainer()->get(SendersLocatorInterface::class);
    static::assertInstanceOf(SendersLocator::class, $sendersLocator);
    $envelope = new Envelope(new SmTestMessage());
    $senders = iterator_to_array($sendersLocator->getSenders($envelope));
    static::assertCount(0, $senders);

    // Fallback locator.
    $this->setRouting(['*' => 'sm_test_transport__in_memory']);
    $definition = $this->container->getDefinition(SendersLocatorInterface::class);
    static::assertEquals([
      '*' => ['sm_test_transport__in_memory'],
    ], $definition->getArgument(0));

    $arg1 = $definition->getArgument(1);
    static::assertInstanceOf(Reference::class, $arg1);
    $serviceLocator = $this->container->get((string) $arg1);
    static::assertInstanceOf(ServiceLocator::class, $serviceLocator);
    static::assertEquals([
      'sm_test_transport.transport.sm_test_transport__in_memory',
      'sm_test_transport.transport.sm_test_transport__in_memory2',
      'sm_test_transport__in_memory',
      'sm_test_transport__in_memory2',
    ], array_keys($serviceLocator->getProvidedServices()));

    $sendersLocator = \Drupal::getContainer()->get(SendersLocatorInterface::class);
    static::assertInstanceOf(SendersLocator::class, $sendersLocator);
    $envelope = new Envelope(new SmTestMessage());
    $senders = iterator_to_array($sendersLocator->getSenders($envelope));
    static::assertCount(1, $senders);
    static::assertArrayHasKey('sm_test_transport__in_memory', $senders);
    static::assertInstanceOf(InMemoryTransport::class, $senders['sm_test_transport__in_memory']);
  }

  /**
   * Tests matching on partial class name.
   *
   * This is functionality provided by Symfony Messenger.
   */
  public function testSendersLocatorPartialClassMatch(): void {
    $this->setRouting([
      'Drupal\sm_test\*' => 'sm_test_transport__in_memory2',
      '*' => 'sm_test_transport__in_memory',
    ]);

    $sendersLocator = \Drupal::getContainer()->get(SendersLocatorInterface::class);
    $envelope = new Envelope(new SmTestMessage());
    $senders = iterator_to_array($sendersLocator->getSenders($envelope));
    static::assertCount(1, $senders);
    static::assertArrayHasKey('sm_test_transport__in_memory2', $senders);
  }

  /**
   * Tests matching on exact class name.
   *
   * This is functionality provided by Symfony Messenger.
   */
  public function testSendersLocatorExactClassMatch(): void {
    $this->setRouting([
      'Drupal\sm_test\SmTestMessage' => 'sm_test_transport__in_memory2',
      'Drupal\sm_test' => 'sm_test_transport__in_memory',
      '*' => 'sm_test_transport__in_memory',
    ]);

    $sendersLocator = \Drupal::getContainer()->get(SendersLocatorInterface::class);
    $envelope = new Envelope(new SmTestMessage());
    $senders = iterator_to_array($sendersLocator->getSenders($envelope));
    static::assertCount(1, $senders);
    static::assertArrayHasKey('sm_test_transport__in_memory2', $senders);
  }

  /**
   * {@inheritdoc}
   */
  public function register(ContainerBuilder $container): void {
    // This method can be invoked again when calling kernel->rebuildContainer
    // a strategy borrowed from `DrupalFlushAllCachesTest`.
    parent::register($container);

    if (NULL !== $this->messageToSendersMapping) {
      $container->setParameter('sm.routing', $this->messageToSendersMapping);
    }
  }

  /**
   * Sets routing and triggers a rebuild.
   *
   * @phpstan-param array<string|class-string, string|string[]> $messageToSendersMapping
   */
  private function setRouting(array $messageToSendersMapping): void {
    $this->messageToSendersMapping = $messageToSendersMapping;
    \Drupal::service('kernel')->rebuildContainer();
  }

}
